package io.skore.datami_plugin

import android.content.Context
import android.content.pm.PackageManager
import android.util.Log
import androidx.annotation.NonNull
import com.datami.smi.SdStateChangeListener
import com.datami.smi.SmiResult
import com.datami.smi.SmiSdk
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar


class DatamiPlugin() : FlutterPlugin, MethodCallHandler, SdStateChangeListener {
    private var channel: MethodChannel? = null
    private val TAG = DatamiPlugin::class.java.simpleName
    private lateinit var context: Context
    override fun onAttachedToEngine(
            @NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding
    ) {
        onAttachedToEngine(flutterPluginBinding.binaryMessenger)
        this.context = flutterPluginBinding.applicationContext
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel?.setMethodCallHandler(null)
        channel = null
    }

    companion object {
        private var instance: DatamiPlugin? = null

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            if (instance == null) {
                instance = DatamiPlugin().apply { onAttachedToEngine(registrar.messenger()) }
            }
        }
    }

    fun onAttachedToEngine(binaryMessenger: BinaryMessenger) {
        if (channel == null) {
            channel =
                    MethodChannel(binaryMessenger, "datami_plugin").apply {
                        setMethodCallHandler(this@DatamiPlugin)
                    }
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "initSponsoredData" -> {
                val smiKey = call.argument<String>("smiKey")
                if (smiKey.isNullOrBlank()) {
                    result.error("ERROR", "Invalid smiKey argument", null)
                    return
                }
                val userID = call.argument<String>("userID") ?: "123"
                Log.d(TAG, "-->> userID: $userID")

                val appIcon =
                        context.packageManager.getApplicationInfo(
                                        context.packageName,
                                        PackageManager.GET_META_DATA
                                )
                                .icon
                SmiSdk.initSponsoredData(smiKey, context, userID, appIcon, true, null)

                result.success(0)
            }
           // CALL ONLY ONCE
            "addSdStateChangeListener" -> {
                SmiSdk.addSdStateChangeListener {
                    val sdState = it.sdState
                    val sdReason = it.sdReason
                    Log.d(TAG, "-->> onChange: $sdState - $sdReason")
                    val handler = android.os.Handler(android.os.Looper.getMainLooper())
                    handler.post(
                        object :java.lang.Runnable {
                            override fun run() {
                                channel?.invokeMethod(
                                        "onSdStateChange",
                                        intArrayOf(sdState.statusCode, sdReason.statusCode))
                            }
                        }
                    )

                }
                result.success(0)
            }
            "startSponsoredData" -> {
                SmiSdk.startSponsoredData()
                result.success(0)
            }
            "stopSponsoredData" -> {
                SmiSdk.stopSponsoredData()
                result.success(0)
            }
            else -> result.notImplemented()
        }
    }
    override fun onChange(currentSmiResult: SmiResult) {
        val sdState = currentSmiResult.sdState
        val sdReason = currentSmiResult.sdReason

        Log.d(TAG, "-->> State: $sdState")
        Log.d(TAG, "-->> Reason: $sdReason")

        channel?.invokeMethod(
                "onSmiStatusChange",
                intArrayOf(sdState.statusCode, sdReason.statusCode)
        )
    }
}
