import Flutter
import UIKit

public class SwiftDatamiPlugin: NSObject, FlutterPlugin {

  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "datami_plugin", binaryMessenger: registrar.messenger())
    let instance = SwiftDatamiPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)

    DatamiService.flutterChannel = channel

  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "initSponsoredData":
      initSponsoredData(arguments: call.arguments, methodResult: result)
    case "addSdStateChangeListener":
      addSdStateChangeListener(methodResult: result)
    case "startSponsoredData":
      startSponsoredData(methodResult: result)
    case "stopSponsoredData":
      stopSponsoredData(methodResult: result)
    default:
      result(FlutterMethodNotImplemented)
    }
  }

  private func startSponsoredData(methodResult: FlutterResult){
    DatamiService.startSponsorData()
    methodResult(0)
  }

  private func stopSponsoredData(methodResult: FlutterResult){
    DatamiService.stopSponsoredData()
    methodResult(0)
  }

  private func initSponsoredData(arguments: Any?, methodResult: FlutterResult) {
    guard let json = arguments as? Dictionary<String, Any> else {
      methodResult(arguments)
      return
    }

    let datamiKey = json["smiKey"] as? String ?? ""
    let userID = json["userID"] as? String ?? ""

    DatamiService.initSponsoredData(datamiKey, userId: userID)
    methodResult(0)
  }

  private func addSdStateChangeListener(methodResult: FlutterResult) {
    DatamiService.registerNotificationObserver()
    methodResult(0)
  }

}
