import 'dart:async';
import 'package:datami_plugin/smi_status.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class DatamiPlugin {
  static const MethodChannel _channel = const MethodChannel('datami_plugin');

  static SmiStatus _getSmiStatus(int status) {
    switch (status) {
      case 1:
        return SmiStatus.wifi;
      case 2:
        return SmiStatus.sd_available;
      case 3:
        return SmiStatus.sd_not_available;
      default:
        return SmiStatus.unknown;
    }
  }

  static Future<bool> initSponsoredData(
      {required String smiKey,
      required String userID,
      onSmiStatusChange}) async {
    try {
      await _channel.invokeMethod('initSponsoredData',
          <String, dynamic>{'smiKey': smiKey, 'userID': userID});
    } on PlatformException catch (e) {
      debugPrint('erro initSponsoredData ${e.message}');
      return false;
    }
    return true;
  }

  static Future<bool> stopSponsoredData() async {
    try {
      await _channel.invokeMethod('stopSponsoredData');
    } on PlatformException catch (e) {
      debugPrint('erro stopDatami ${e.message}');
      return false;
    }
    return true;
  }

  static Future<bool> startSponsoredData() async {
    try {
      await _channel.invokeMethod('startSponsoredData');
    } on PlatformException catch (e) {
      debugPrint('erro startSponsoredData ${e.message}');
      return false;
    }
    return true;
  }

  static Future<bool> addSdStateChangeListener(onSmiStatusChange) async {
    int? result;
    try {
      _initializeChannel(onSmiStatusChange: onSmiStatusChange);
      result = await _channel.invokeMethod('addSdStateChangeListener');
    } on PlatformException catch (e) {
      debugPrint('erro addSdStateChangeListener ${e.message}');
      return false;
    }
    return result == 0;
  }

  static void _initializeChannel({onSmiStatusChange}) {
    _channel.setMethodCallHandler((MethodCall call) async {
      String method = call.method;
      List? arguments = call.arguments;
      if (method == 'onSdStateChange' &&
          (arguments!.first is int) &&
          (arguments.last is int)) {
        int sdState = arguments.first;
        String sdReason = SDReason.status(arguments.last);
        SmiStatus smiStatus = _getSmiStatus(sdState);
        onSmiStatusChange(smiStatus, sdReason);
      }
    });
  }
}

class SDReason {
  static String status(int value) {
    String sdReason;

    switch (value) {
      case 0:
        sdReason = 'SD_REASON_NONE';
        break;
      case 1:
        sdReason = 'SD_REASON_DUPLICATE_API_CALL';
        break;
      case 2:
        sdReason = 'SD_NOT_AVAILABLE_FOR_OPERATOR';
        break;
      case 3:
        sdReason = 'SD_NOT_AVAILABLE_FOR_APPLICATION';
        break;
      case 4:
        sdReason = 'SD_NOT_AVAILABLE_FOR_URL';
        break;
      case 5:
        sdReason = 'SD_NOT_AVAILABLE_FOR_DEVICE_OR_USER';
        break;
      case 6:
        sdReason = 'SD_NOT_AVAILABLE_PROMOTION_EXPIRED';
        break;
      case 7:
        sdReason = 'SD_NOT_AVAILABLE_PROMOTION_SUSPENDED';
        break;
      case 8:
        sdReason = 'SD_NOT_AVAILABLE_PROMOTION_LIMIT_EXCEEDED';
        break;
      case 9:
        sdReason = 'SD_NOT_AVAILABLE_USER_LIMIT_EXCEEDED';
        break;
      case 10:
        sdReason = 'SD_NOT_AVAILABLE_PROMOTION_NOT_FOUND';
        break;
      case 11:
        sdReason = 'SD_NOT_AVAILABLE_SDK_INTERNAL_ERROR';
        break;
      case 12:
        sdReason = 'SD_NOT_AVAILABLE_SDK_VERSION_NOT_SUPPORTED';
        break;
      case 13:
        sdReason = 'SD_NOT_AVAILABLE_SDK_REGISTRATION_NOT_DONE';
        break;
      case 14:
        sdReason = 'SD_NOT_AVAILABLE_SDK_NOT_SUPPORTED_FOR_APP_OR_CARRIER';
        break;
      case 15:
        sdReason = 'SD_NOT_AVAILABLE_URAM_NO_HEADER_INJECTED';
        break;
      case 16:
        sdReason = 'SD_NOT_AVAILABLE_SD_NOT_SUPPORTED_IN_ROAMING';
        break;
      case 17:
        sdReason = 'SD_NOT_AVAILABLE_SD_TESTING';
        break;
      case 18:
        sdReason = 'SD_NOT_AVAILABLE_APP_API_ENABLED';
        break;
      case 19:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONNECTION_FAILURE';
        break;
      case 20:
        sdReason = 'SD_NOT_AVAILABLE_NO_DATA_CONNECTION';
        break;
      case 21:
        sdReason = 'SD_NOT_AVAILABLE_CONNECTION_TIMEOUT';
        break;
      case 22:
        sdReason = 'SD_NOT_AVAILABLE_CONNECTION_LOST';
        break;
      case 23:
        sdReason = 'SD_NOT_AVAILABLE_DNS_ERROR';
        break;
      case 24:
        sdReason = 'SD_NOT_AVAILABLE_APP_TRANSPORT_SECURITY_ERROR';
        break;
      case 25:
        sdReason = 'SD_NOT_AVAILABLE_SD_STOP_API_CALL';
        break;
      case 26:
        sdReason = 'SD_NOT_AVAILABLE_ZERO_BALANCE_CAMPAIGN';
        break;
      case 27:
        sdReason = 'SD_AVAILABLE_ONLY_FOR_ZERO_BALANCE_USERS';
        break;
      case 28:
        sdReason = 'SD_NOT_AVAILABLE_GW_HEALTH_CHECK_FAILURE';
        break;
      case 29:
        sdReason = 'SD_NOT_AVAILABLE_GW_FAILURE_MULTIPLE_CLEAR_CACHE';
        break;
      case 30:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONN_FAILURE_USR_DLY_FUP_REACHED';
        break;
      case 31:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONN_FAILURE_USR_WKLY_FUP_REACHED';
        break;
      case 32:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONN_FAILURE_USR_MTHLY_FUP_REACHED';
        break;
      case 33:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONN_FAILURE_USR_OVR_ALL_FUP_REACHED';
        break;
      case 34:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONN_FAILURE_PKG_DLY_FUP_REACHED';
        break;
      case 35:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONN_FAILURE_PKG_WKLY_FUP_REACHED';
        break;
      case 36:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONN_FAILURE_PKG_MTHLY_FUP_REACHED';
        break;
      case 37:
        sdReason = 'SD_NOT_AVAILABLE_GW_CONN_FAILURE_PKG_OVR_ALL_FUP_REACHED';
        break;
      case 38:
        sdReason = 'SD_NOT_AVAILABLE_VPN_PERMISSION_CANCELLED';
        break;
      case 100:
        sdReason = 'SD_NOT_AVAILABLE_REASON_UNKNOWN';
        break;
      default:
        sdReason = 'NOT_FOUND';
    }

    return sdReason;
  }
}
